@@ -1,15 +1,15 @@
-from mpi4py.futures import MPIPoolExecutor
-from mpi4py import MPI
-import cloudpickle as pickle
+"""Engines with multi-node parallelization."""
 import logging
-from tqdm import tqdm
-
 from typing import List
 
+import cloudpickle as pickle
+from mpi4py import MPI
+from mpi4py.futures import MPIPoolExecutor
+from tqdm import tqdm
+
 from .base import Engine
 from .task import Task
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -21,9 +21,9 @@ def work(pickled_task):
 
 class MPIPoolEngine(Engine):
     """
-    Parallelize the task execution using
-    `mpi4py <https://mpi4py.readthedocs.io/en/stable/>`_.
+    Parallelize the task execution.
 
+    Uses `mpi4py <https://mpi4py.readthedocs.io/en/stable/>`_.
     To be called with:
     ``mpiexec -np #Workers+1 python -m mpi4py.futures YOURFILE.py``
     """
@@ -32,7 +32,8 @@ class MPIPoolEngine(Engine):
         super().__init__()
 
     def execute(self, tasks: List[Task], progress_bar: bool = True):
-        """Pickle tasks and distribute work to workers.
+        """
+        Pickle tasks and distribute work to workers.
 
         Parameters
         ----------
@@ -41,16 +42,16 @@ class MPIPoolEngine(Engine):
         progress_bar:
             Whether to display a progress bar.
         """
-
         pickled_tasks = [pickle.dumps(task) for task in tasks]
 
-        n_procs = MPI.COMM_WORLD.Get_size()   # Size of communicator
-        logger.info(f"Performing parallel task execution on {n_procs-1} "
-                    f"workers with one manager.")
+        n_procs = MPI.COMM_WORLD.Get_size()  # Size of communicator
+        logger.info(
+            f"Performing parallel task execution on {n_procs-1} "
+            f"workers with one manager."
+        )
 
         with MPIPoolExecutor() as executor:
-            results = executor.map(work,
-                                   tqdm(pickled_tasks,
-                                        disable=not progress_bar)
-                                   )
+            results = executor.map(
+                work, tqdm(pickled_tasks, disable=not progress_bar)
+            )
         return results
