@@ -1,14 +1,15 @@
-from concurrent.futures import ThreadPoolExecutor
+"""Engines with multi-threading parallelization."""
 import copy
-import os
 import logging
-from tqdm import tqdm
+import os
+from concurrent.futures import ThreadPoolExecutor
 from typing import List
 
+from tqdm import tqdm
+
 from .base import Engine
 from .task import Task
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -39,7 +40,8 @@ class MultiThreadEngine(Engine):
             logger.warning(
                 f"Engine set up to use up to {n_threads} processes in total. "
                 f"The number was automatically determined and might not be "
-                f"appropriate on some systems.")
+                f"appropriate on some systems."
+            )
         self.n_threads: int = n_threads
 
     def execute(self, tasks: List[Task], progress_bar: bool = True):
@@ -57,13 +59,13 @@ class MultiThreadEngine(Engine):
         copied_tasks = [copy.deepcopy(task) for task in tasks]
 
         n_threads = min(self.n_threads, n_tasks)
-        logger.info(f"Performing parallel task execution on {n_threads} "
-                    f"threads.")
+        logger.info(
+            f"Performing parallel task execution on {n_threads} " f"threads."
+        )
 
         with ThreadPoolExecutor(max_workers=n_threads) as pool:
-            results = pool.map(work,
-                               tqdm(copied_tasks,
-                                    disable=not progress_bar)
-                               )
+            results = pool.map(
+                work, tqdm(copied_tasks, disable=not progress_bar)
+            )
 
         return results
