@@ -1,6 +1,7 @@
 import logging
 from typing import Sequence, Tuple
 
+from ..C import ModeType
 from ..engine import Task
 
 logger = logging.getLogger(__name__)
@@ -26,12 +27,12 @@ class PredictorTask(Task):
     """
 
     def __init__(
-            self,
-            predictor: 'pypesto.predict.Predictor',  # noqa: F821
-            x: Sequence[float],
-            sensi_orders: Tuple[int, ...],
-            mode: str,
-            id: str,
+        self,
+        predictor: 'pypesto.predict.Predictor',  # noqa: F821
+        x: Sequence[float],
+        sensi_orders: Tuple[int, ...],
+        mode: ModeType,
+        id: str,
     ):
         super().__init__()
         self.predictor = predictor
@@ -41,6 +42,7 @@ class PredictorTask(Task):
         self.id = id
 
     def execute(self) -> 'pypesto.predict.PredictionResult':  # noqa: F821
+        """Execute and return the prediction."""
         logger.info(f"Executing task {self.id}.")
         prediction = self.predictor(self.x, self.sensi_orders, self.mode)
         return prediction
